// P-Channel MOSFET Tester
// 
// Left Button Drives Gate Pin (A0)
/////////////////////////////////////////////////

#include "ssd1306.h"

const PROGMEM uint8_t pmosfet[] ={ 
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 
  0x00, 0x80, 0x80, 0x00,  0x00, 0x00, 0x00, 0x00, 
  0x3f, 0x21, 0x21, 0x21,  0x1e, 0x00, 0xff, 0xff, 

  0xe6, 0x16, 0x96, 0x96,  0xa6, 0x06, 0xfe, 0xfe, 
  0x00, 0xdf, 0xdf, 0x06,  0xe6, 0xc6, 0x86, 0x06, 
  0x06, 0x06, 0x06, 0x06,  0x06, 0x06, 0x07, 0x07, 

  0x01, 0x02, 0x02, 0x02,  0x01, 0x00, 0xff, 0xff, 
  0x00, 0xef, 0xef, 0x83,  0x9f, 0x8f, 0x87, 0x83, 
  0x83, 0x83, 0x83, 0x83,  0x83, 0x83, 0xff, 0xff, 

  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x01, 0x01, 
  0x00, 0x07, 0x07, 0x01,  0x01, 0x01, 0x01, 0x01, 
  0x49, 0x95, 0x95, 0x95,  0x65, 0x01, 0xff, 0xff
};

#define A0_680R 9
#define A0_470K 8
#define A1_680R 7
#define A1_470K 6
#define A2_680R 4
#define A2_470K 3

#define LeftButton 12

void setup() {
  ssd1306_setFixedFont(ssd1306xled_font6x8);
  ssd1306_128x64_i2c_init();
  ssd1306_clearScreen();
  
  pinMode(LeftButton, INPUT_PULLUP);
}

void loop() {
  float VRead;
  int Button;
  char strBuf[20];

  // A2 (SOURCE PIN) connects directly to 5V
  pinMode(A2, OUTPUT); 
  pinMode(A2_470K, INPUT); 
  pinMode(A2_680R, INPUT); 
  digitalWrite(A2, 1);

  // A1 (DRAIN PIN) pulled-down by 470K to GND
  pinMode(A1_680R, INPUT); 
  pinMode(A1, INPUT); 
  pinMode(A1_470K, OUTPUT);
  digitalWrite(A1_470K, 0);

  // A0 (GATE PIN) connects to 5V or GND as control
  pinMode(A0, OUTPUT);
  pinMode(A0_680R, INPUT); 
  pinMode(A0_470K, INPUT);  

  ssd1306_printFixed(0,  0, "P-Channel MOSFET", STYLE_NORMAL);
  ssd1306_drawBitmap(10, 10, 24, 32, pmosfet);

  while (1) {
    Button = digitalRead(LeftButton);
    digitalWrite(A0, Button);
    if (Button)
      ssd1306_printFixed(55,  35, "Gate: OPEN  ", STYLE_NORMAL);
    else
      ssd1306_printFixed(55,  35, "Gate: CLOSED", STYLE_NORMAL);
      
    VRead = analogRead(A1);
    VRead = 5 * (VRead / 1023);
    sprintf(strBuf, "Vout: %dV", (long)VRead); 
    ssd1306_printFixed(55,  50, strBuf, STYLE_NORMAL);
    
    delay(250);
  }

}
